require 'class'
require 'utils'

local floor = math.floor

local draw = love.graphics.draw
local gprint, gprintf = love.graphics.print, love.graphics.printf
local newImage = love.graphics.newImage
local getMouseX, getMouseY = love.mouse.getX, love.mouse.getY
local rect, line = love.graphics.rectangle, love.graphics.line
local setColor, getColor = love.graphics.setColor, love.graphics.getColor
local setFont, getFont, newFont = love.graphics.setFont, love.graphics.getFont, love.graphics.newFont
local CS = 16

LoveView = class()

LoveView.load = function(self)
	love.mouse.setVisible(false)

	self.panels = newImage('img/panels.png')
	self.selection_box = newImage('img/selection_box.png')
	self.selection_panel = newImage('img/selection_panel.png')
	self.common_panel = newImage('img/common_panel.png')
	self.keyboard_panel = newImage('img/keyboard_panel.png')
	self.menu_panel = newImage('img/menu_panel.png')

	self.font = newFont('04B_03__.TTF', 8)
	setFont(self.font)
end

LoveView.update = function(self, dt)
end

LoveView.draw = function(self, model)
	local line = model.line
	local caret = model.caret

	line = '>'..line:sub(1,caret-1) .. '|' ..line:sub(caret)

	gprintf(line, 0, 0 , 320, 'left')

	draw(self.panels, 0, 10*CS)

	if model.panel == 1 then
		draw(self.common_panel, 0, 11*CS)
	elseif model.panel == 2 then
		draw(self.keyboard_panel, 0, 11*CS)
	elseif model.panel == 3 then
		draw(self.menu_panel, 0, 11*CS)
	end

	draw(self.selection_panel, 4*CS*(model.panel-1), 10*CS)

	if model.focus == 'pad' then
		local x, y = model.box[2], model.box[1]
		draw(self.selection_box, (x-1)*CS, (y-1)*CS + 11*CS)
	end
end
