require 'class'

local exit = function() love.event.push('q') end

local CS = 16
local MINX, MAXX = 1, 20
local MINY, MAXY = 1, 4

local buttonphrases = require 'buttonphrases'
local help = require 'help'

Model = class()

Model.load = function(self)
	self.panel = 1
	self.select = false
	self.panelkeys = {'common', 'keyboard', 'menu'}
	self.panels = {
		common = require 'commonpanel',
		keyboard = require 'keyboardpanel',
		menu = require 'menupanel'}
	self.box = {1,1}
	self.caret = 1
	self.line = help
	self.row = 0
	self.firstrow = 1
	self.lastrow = 0
	self.text = {}
	self.savedbuffer = 0
	self.buffersize = 64
	self.error = true

	self.focus = 'textbox'
end

Model.move = function(self, dir)
	if self.focus == 'textbox' then
		self:moveCaret(dir)
	elseif self.focus == 'pad' then
		self:moveSelection(dir)
	elseif self.focus == 'tabs' then
		self:movePanel(dir)
	end
end

Model.movePanel = function(self, dir)
	if dir == 'right' then
		if self.panel + 1 > #self.panelkeys then
			self.panel = 1
		else
			self.panel = self.panel + 1
		end
	elseif dir == 'left' then
		if self.panel - 1 < 1 then
			self.panel = #self.panelkeys
		else
			self.panel = self.panel - 1
		end
	end
end

Model.update = function(self, dt)
end


Model.moveSelection = function(self, dir)
	if dir == 'right' then
		self.box[2] = (self.box[2] >= MAXX) and MINX or self.box[2] + 1
	elseif dir == 'left' then
		self.box[2] = (self.box[2]-1 < MINX) and MAXX or self.box[2] - 1
	elseif dir == 'up' then
		self.box[1] = (self.box[1]-1 < MINY) and MAXY or self.box[1] - 1
	elseif dir == 'down' then
		self.box[1] = (self.box[1] > MAXY) and MINY or self.box[1] + 1
	end
end

Model.moveCaret = function(self, dir)
	if dir == 'left' and self.caret > 1 then
		self.caret = self.caret - 1
	elseif dir == 'right' and self.caret < #self.line + 1 then
		self.caret = self.caret + 1
	elseif dir == 'up' and self.lastrow > 0 then
		if self.row-1 < self.firstrow then
			self.row = self.lastrow
		else
			self.row = self.row - 1
		end
		self.line = self.text[self.row]
	elseif dir == 'down' and self.lastrow > 0 then
		if self.row+1 > self.lastrow then
			self.row = self.firstrow
		else
			self.row = self.row + 1
		end
		self.line = self.text[self.row]
	end
end

Model.jump = function(self, label)
	local xy = self.panels[self.panelkeys[self.panel]].labels[label]
	if xy then
		self.box[1], self.box[2] = xy[1], xy[2]
	end
end

Model.processCommand = function(self, command)
	local s = command

	if s == '$backspace' then
		if self.caret > 1 then
			self.line = self.line:sub(1,self.caret-2) .. self.line:sub(self.caret)
			self.caret = self.caret - 1
		end
	elseif s == '$return' then


		local line = self.line
		local eval = false

		if line:sub(1,1) == '=' then
			line = '__result__' .. line
			eval = true
		end

		if line:sub(#line, #line) == ';' then
			eval = true
		end

		if eval then
			local f,e = loadstring(line)

			if f then
				self.lastrow = self.lastrow + 1
				if self.lastrow > self.buffersize then
					self.text[self.firstrow] = nil
					self.firstrow = self.firstrow + 1
				end
				self.text[self.lastrow] = self.line
				self.row = self.lastrow

--~ 				print('LINE', line)
				pcall(f)
--~ 				self.error = pcall(f)

				if self.line:sub(1,1) == '=' then
					self.line = '=' .. tostring(__result__)
					self.caret = #self.line
				else
					self.line = ''
				end
			end

			if self.error then
				self.line = tostring(e or 'Protected call fail')
				self.error = true
			end
		else
			self.error = false
			self.line = self.line:sub(1,self.caret-1) .. '\n' .. self.line:sub(self.caret)
			self.caret = self.caret + 1
		end
	elseif s == '$clear' then
		self.line = ''
		self.error = false
		self.caret = 1
	elseif s == '$space' then
		self.line = self.line:sub(1,self.caret-1) .. ' ' .. self.line:sub(self.caret)
		self.error = false
		self.caret = self.caret + 1
	elseif s == '$$' then
		self.line = self.line:sub(1,self.caret-1) .. '$' .. self.line:sub(self.caret)
		self.error = false
		self.caret = self.caret + 1
	elseif s == '$about' then
		self.line = 'LoveCalc v 0.5.1 by Abraham Greed\nMade for Dingoo A320 in LOVE(obviously)\n'
	elseif s == '$help' then
		self.caret = 1
		self.line = help
	elseif s == '$exit' then
		exit()
	elseif s == '$abutton' then
		self.caret = 1
		self.line = buttonphrases[math.random(#buttonphrases)]
	end
end

Model.processSelection = function(self)
	if self.error then
		self.line = ''
		self.error = false
	end

--~ 	if self.select then
		local s = (self.panels[self.panelkeys[self.panel]][self.box[1]] or {})[self.box[2]]
		if s then
			if s:sub(1,1) == '$' then
				self:processCommand(s)
			else
				self.line = self.line:sub(1,self.caret-1) .. s .. self.line:sub(self.caret)
				self.caret = self.caret + 1
			end
		end
--~ 	end
end

Model.nextPanel = function(self)
	if self.panel + 1 > #self.panelkeys then
		self.panel = 1
	else
		self.panel = self.panel + 1
	end
end
