string.split = function(self, sSeparator, nMax, bRegexp)
	assert(sSeparator ~= '')
	assert(nMax == nil or nMax >= 1)

	local aRecord = {}

	if self:len() > 0 then
		local bPlain = not bRegexp
		nMax = nMax or -1

		local nField=1 nStart=1
		local nFirst,nLast = self:find(sSeparator, nStart, bPlain)
		while nFirst and nMax ~= 0 do
			aRecord[nField] = self:sub(nStart, nFirst-1)
			nField = nField+1
			nStart = nLast+1
			nFirst,nLast = self:find(sSeparator, nStart, bPlain)
			nMax = nMax-1
		end
		aRecord[nField] = self:sub(nStart)
	end

	return aRecord
end


parseSheetXML = function(sheetxml)
	local sheet = love.filesystem.lines( sheetxml or 'sheet.xml' )

	local parse = function(s)
		return s:sub(s:find('"')+1, s:find('"', s:find('"')+1)-1)
	end

	local quads = {}
	local s, e, _, t, x
	local path

	for line in sheet do
		if not path then
			s, e = line:find('<sprite file="')
			if s and e then
				x = line:find('"', line:find('"')+1)
				path = line:sub(e+1, x-1)
			end
		else
			_, s = line:find('<frame ')
			e = line:find(' />')
			if s and e then
				t = line:sub(s+1, e-1):split(' ')
				quads[parse(t[1])] = {tonumber(parse(t[2])),
					tonumber(parse(t[3])),
					tonumber(parse(t[4])),
					tonumber(parse(t[5]))}
			end
		end
	end

	return quads, path
end

isInsideBBox = function(x, y, bbox_x, bbox_y, bbox_width, bbox_height)
	local bbox_x = bbox_x
	if type(bbox_x) == 'table' then
		return (x >= bbox_x[1]) and (x <= (bbox_x[1]+bbox_x[3]) ) and (y >= bbox_x[2]) and (y <= (bbox_x[2] + bbox_x[4]) )
	else
		return (x >= bbox_x) and (x <= (bbox_x+bbox_width) ) and (y >= bbox_y) and (y <= (bbox_y + bbox_height) )
	end
end

newMatrix = function(rows, cols, init)
	assert(rows)
	assert(cols)

	local m = {}
	for x = 1, rows do
		m[x] = {}
		for y = 1, cols do
			m[x][y] = init
		end
	end

	return m
end

initEntity = function(entity, prototype, variables)
	entity.prototype = prototype
	entity.variables = {}
	for k, v in pairs(prototype.variables or {}) do
		entity.variables[k] = v
	end
	for k, v in pairs(variables or {}) do
		entity.variables[k] = v
	end
end
